/**
 * Copyright (c) 2008 Washington University
 */
package org.nrg.xnat.filestore;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 *
 */
public final class Utils {
    private static final Collection<String> BAD_SEGMENTS = new HashSet<String>(Arrays.asList(new String[]{".", ".."}));

    // Pattern for valid local store URIs, based on definitions in RFC 3986
    // segment = *store-char
    // store-uri = segment 
    // store-char = unreserved / sub-delims / ":" / "@"
    private static final Pattern VALID_LOCAL_URI_SEGMENT;
    static {
	final String safeChars = "-.~"		// RFC 3986 unreserved not covered in \w
	    + "!$&'()*+,;="			// RFC 3986 sub-delims
	    + ":@";				// safe RFC 3986 gen-delims

	final StringBuilder sb = new StringBuilder("[\\w\\-\\.\\~");
	for (final char c : safeChars.toCharArray()) {
	    sb.append("\\");
	    sb.append(c);
	}
	sb.append("]+");

	VALID_LOCAL_URI_SEGMENT = Pattern.compile(sb.toString());
    }

    /**
     * Tests whether the given path segment is acceptable.
     * To be acceptible, the segment must be a valid URI segment
     * (as defined in RFC 3986), except that pct-encoded characters
     * are not allowed, and neither are segment names that might
     * have special meaning to the filesystem, such as "." or ".."
     * @param segment Proposed (virtual) directory or file name
     * @return true if valid, false otherwise
     */
    public static boolean isValidPathSegment(final String segment) {
	if (null == segment || BAD_SEGMENTS.contains(segment)) {
	    return false;
	}
	final Matcher m = VALID_LOCAL_URI_SEGMENT.matcher(segment);
	return m.matches();
    }
    
    
    /**
     * Tests whether the given path is acceptable, i.e., composed of
     * only valid path segments separated by "/"
     * @param path Proposed path
     * @return true if valid, false otherwise
     */
     public static boolean isValidPath(final String path) {
	if (null == path) {
	    return false;
	}

	final String[] segments = path.split("/");
	if (0 == segments.length) {
	    return false;
	}

	for (final String segment : segments) {
	    if (!isValidPathSegment(segment)) {
		return false;
	    }
	}
	return true;
    }
}
